﻿--[[

************************************************************************

AckisRecipeList
$Date: 2008-10-08 21:10:25 +0000 (Wed, 08 Oct 2008) $
$Rev: 925 $

Author: Ackis on Illidan US Horde
GUI done by Zhinjio

************************************************************************

Please see Wowace.com for more information.

Documentation:
	Please see Documentation.txt for comprehensive documentation.

************************************************************************

--]]


AckisRecipeList = LibStub("AceAddon-3.0"):NewAddon("Ackis Recipe List", "AceConsole-3.0", "AceEvent-3.0")

local MODNAME			= "Ackis Recipe List"

local addon				= LibStub("AceAddon-3.0"):GetAddon(MODNAME)

local BFAC				= LibStub("LibBabble-Faction-3.0"):GetLookupTable()
local L					= LibStub("AceLocale-3.0"):GetLocale(MODNAME)


-- We should probably clean thes up and use arguments/return values
-- Global variables which are used between multiple files
addon.SkillType = nil

addon.playerProfession = ""
addon.MaxFilterIndex = 129

addon.optionsFrame = {}

-- Frame variables
addon.ScanButton = nil
addon.Frame = nil

-- tooltip needs to be global so I can access individual lines in it and 
-- modify then
_G["arlTooltip"]= nil

-- Make global API calls local to speed things up
local GetNumTradeSkills = GetNumTradeSkills
local GetSpellInfo = GetSpellInfo
local GetSpellName = GetSpellName
local GetTradeSkillLine = GetTradeSkillLine
local GetTradeSkillInfo = GetTradeSkillInfo

local select = select
local format = format
local string = string
local tostring = tostring
local tonumber = tonumber
local pairs = pairs
local table = table
local next = next
local tremove = table.remove
local tsort = table.sort
local tinsert = table.insert
local sfind = string.find
local InterfaceOptionsFrame_OpenToFrame = InterfaceOptionsFrame_OpenToFrame
local BOOKTYPE_SPELL = BOOKTYPE_SPELL

-- Constants which are used everytime the add-on is loaded

-- New Shiny addonversion without Revision
addon.addonversion = GetAddOnMetadata("AckisRecipeList", "Version")

-- Global constants which are used between multiple files
addon.ARLTitle = "ARL (v." .. addon.addonversion .. ")"
addon.FullTitle = "Ackis Recipe List (v." .. addon.addonversion .. ")"

-- We use line breaks quite often lets make it a constant send you can't use \t in WoW
addon.br = "\n    - " -- Nuke this

--[[

	Initialization functions

]]--

-- Description: Function run when the addon is initialized.  Registers the slash commands, options, and database
-- Expected result: The addon parameters are initialized, slash commands are enabled, database is enabled, and
--	interface options are registered
-- Input: None
-- Output: None

function addon:OnInitialize()

	self.db = LibStub("AceDB-3.0"):New("ARLDB2")

	self:SetupOptions()

	-- Register slash commands
	self:RegisterChatCommand("arl", "ChatCommand")
	self:RegisterChatCommand("ackisrecipelist", "ChatCommand")

	-- Set default options, which are to include everything in the scan
	self.db:RegisterDefaults(
	{
		profile = {
			-- Sorting Options
			sorting = L["Skill"],

			-- Display Options
			includefiltered = false,
			closeguionskillclose = false,
			ignoreexclusionlist = false,
			uiscale = 1,
			tooltipscale = .9,

			-- Recipe Exclusion
			exclusionlist = {},

			-- Filter Options
			filters = {
				general = {			-- General Filters
				    faction = true,
					class = false,
					specialty = false,
					skill = true,
					known = false,
					unknown = true,
				},
				obtain = {			-- Obtain Options
					trainer = true,
					vendor = true,
					instance = true,
					raid = true,
					seasonal = true,
					quest = true,
					pvp = true,
					discovery = true,
					worlddrop = true,
					mobdrop = true,
				},
				item = {			-- Item Filters (Armor/Weapon)
					armor = {
						cloth = true,
						leather = true,
						mail = true,
						plate = true,
						trinket = true,
						cloak = true,
						ring = true,
						necklace = true,
					},
					weapon = {
						onehand = true,
						twohand = true,
						axe = true,
						sword = true,
						mace = true,
						polearm = true,
						dagger = true,
						fist = true,
						staff = true,
						wand = true,
						thrown = true,
						bow = true,
						crossbow = true,
						ammo = true,
					},
				},
				binding = {
					itemboe = true,
					itembop = true,
					recipebop = true,
					recipeboe = true,
				},
				player = {
					melee = true,
					tank = true,
					healer = true,
					caster = true,
				},
				rep = {			-- Reputation Options
					aldor = true,
					scryer = true,
					argentdawn = true,
					ashtonguedeathsworn = true,
					cenarioncircle = true,
					cenarionexpedition = true,
					consortium = true,
					hellfire = true,
					keepersoftime = true,
					nagrand = true,
					lowercity = true,
					scaleofthesands = true,
					shatar = true,
					shatteredsun = true,
					sporeggar = true,
					thoriumbrotherhood = true,
					timbermaw = true,
					violeteye = true,
					zandalar = true,
-- Below this is WotLK stuff. May need to be changed post beta.
					argentcrusade = true,
					frenzyheart = true,
					ebonblade = true,
					kirintor = true,
					sonsofhodir = true,
					kaluak = true,
					oracles = true,
					wyrmrest = true,
					silvercovenant = true,
					sunreavers = true,
					explorersleague = true,
					valiance = true,
					handofvengeance = true,
					taunka = true,
					warsongoffensive = true,
				}
			}
		}
	} )

end

-- Description: Function run when the addon is enabled.  Registers events and pre-loads certain variables.
-- Expected result: Events are registered, scan button and faction levels are identified
-- Input: None
-- Output: None

function addon:OnEnable()

	-- Make addon respond to the tradeskill windows being shown
	self:RegisterEvent("TRADE_SKILL_SHOW")

	-- Addon responds to tradeskill windows being closed.
	self:RegisterEvent("TRADE_SKILL_CLOSE")

	-- Addon responds when a players faction level changes
	self:RegisterEvent("UPDATE_FACTION")

	-- Add an option so that ARL will work with Manufac
	if (Manufac) then
		Manufac.options.args.ARLScan = {
			type = 'execute',
			name = L["Scan Skills"],
			desc = L["Scan Skills Long"],
			func = function() addon:AckisRecipeList_Command() end,
			order = 550,
		}
	end

	-- Populate the repuatation level
	self:GetFactionLevels()

	--Create the button now for later use
	self:CreateScanButton()

end

-- Description: Run when the addon is disabled. Ace3 takes care of unregistering events, etc.
-- Expected result: Events are unregistered, frame is hidden if it exists and plug-ins into other mods are removed
-- Input: None
-- Output: None

function addon:OnDisable()

	-- If we disable the addon when the GUI is up, hide it.
	if (addon.Frame) then
		addon.Frame:Hide()
	end

	-- Remove the option from Manufac
	if Manufac then
		Manufac.options.args.ARLScan = nil
	end

end

-- Description: Shows the scan button when the trade skill window is open.
-- Expected result: Scan button is shown.
-- Input: None
-- Output: None

--[[

	Event handling functions

]]--

function addon:TRADE_SKILL_SHOW()

	addon.SkillType = "Trade"

	if (addon.ScanButton and not Skillet) then
		self:ShowScanButton()
	end

end

-- Description: Hides the scan button when the trade skill window is closed.
-- Expected result: Scan button is hidden and the GUI is closed if the person has that option selected.
-- Input: None
-- Output: None

function addon:TRADE_SKILL_CLOSE()

	addon.SkillType = nil

	if (addon.db.profile.closeguionskillclose and addon.Frame) then
		self:CloseWindow()
	end

	if (addon.ScanButton and not Skillet) then
		addon.ScanButton:Hide()
	end

end

-- Description: Will update the internal list of faction tables when a players faction changes
-- Expected result: Faction table is updated with new reputation levels
-- Input: None
-- Output: None

function addon:UPDATE_FACTION()

	-- Reputation has changed so lets update the table
	self:SetRepDB()

end

--[[

	Player Data Acquisition Functions

]]--

-- Description: Scans all reputations to get reputation levels to determine if the player can learn a reputation recipe
-- Expected result: Updates a table with players current faction level
-- Input: Faction table
-- Output: None, array is referenced

function addon:GetFactionLevels(RepTable)
-- Bug here when I reload UI
	if (not RepTable) then
		return
	end

	for i=1,GetNumFactions(),1 do
		local name,_,replevel = GetFactionInfo(i)

		-- If the rep is greater than neutral
		if (replevel > 4) then
		-- We use levels of 0, 1, 2, 3, 4 internally for reputation levels, make it corrospond here
			RepTable[name] = replevel - 4
		end
	end
end

-- Description: Scans first 25 spellbook slots to identify all applicable professions
-- Expected result: Updates a table with players current professions
-- Input: Profession table
-- Output: None, array is referenced

function addon:GetKnownProfessions(ProfTable)

	-- Reset the table, they may have unlearnt a profession
	for i in pairs(ProfTable) do

		ProfTable[i] = false

	end

	-- Scan through the spell book getting the spell names
	for index=1,25,1 do

		local spellName = GetSpellName(index, BOOKTYPE_SPELL)

		if (not spellName) or (index == 25) then

			-- Nothing found
			break

		end

		if (ProfTable[spellName] == false) then

			ProfTable[spellName] = true

		end
	end

end

-- Description: Scans first 25 spellbook slots to identify which trade skill Specialty we have
-- Expected result: Returns the name of the trade skill Specialty
-- Input: Specialty Table and Current Profession
-- Output: String of current Specialty

function addon:GetTradeSpecialty(SpecialtyTable, playerdata)

	--Scan the first 25 entries
	for index=1,25,1 do

		local spellName = GetSpellName(index, BOOKTYPE_SPELL)

		-- Nothing found, return nothing
		if (not spellName) or (index == 25) or (not SpecialtyTable[playerdata.playerProfession]) then

			return ""

		-- We have a match, return that spell name
		elseif (SpecialtyTable[playerdata.playerProfession][spellName]) then

			return spellName

		end

	end

end

--[[
	Tradeskill functions
--]]

-- Description: Adds a specific recipe, along with it's info to an array
-- Expected result: The RecipeDB will have a recipe entry created for it.
-- Input: Recipe array reference, SpellID of recipe, Skill level of recipe, Item ID of item created, Rarity of recipe, and Specialty of the recipe
-- Output: None, array is passed as a reference

function addon:addTradeSkill(RecipeDB, SpellID, SkillLevel, ItemID, Rarity, Profession, Specialty)

	--[[ 
		Recipe DB Structures are defined in Documentation.lua
	--]]
	
	-- Creates a table in the RecipeListing table storing all information about a recipe
	RecipeDB[SpellID] = {}
	-- Set the information passed
	RecipeDB[SpellID]["Level"] = SkillLevel
	RecipeDB[SpellID]["ItemID"] = ItemID
	RecipeDB[SpellID]["Rarity"] = Rarity
	RecipeDB[SpellID]["Profession"] = GetSpellInfo(Profession)

	-- Get the recipe link from the spell ID
	local spellLink = GetSpellLink( SpellID )
	if ( spellLink ~= nil ) then
		RecipeDB[SpellID]["RecipeLink"] = string.gsub( spellLink, "spell", "enchant")
	else
		RecipeDB[SpellID]["RecipeLink"] = nil
	end

	-- Get the recipe name now
	RecipeDB[SpellID]["Name"] = GetSpellInfo(SpellID) or nil

	if (RecipeDB[SpellID]["Name"] == nil) then

		self:Print(format(L["SpellIDCache"],SpellID))

	end

	-- All recipes are unknown until scan occurs
	RecipeDB[SpellID]["Known"] = false

	-- All recipes are set to be displayed until the filtering occurs
	RecipeDB[SpellID]["Display"] = true

	-- All recipes are set to be showing in the search results
	RecipeDB[SpellID]["Search"] = true

	-- Create the flag space in the RecipeDB
	RecipeDB[SpellID]["Flags"] = {}

	-- Set all the flags to be false, will also set the padding spaces to false as well.
	for i=1,addon.MaxFilterIndex,1 do

		RecipeDB[SpellID]["Flags"][tonumber(i)] = false

	end

	-- Create the Acquire space in the RecipeDB
	RecipeDB[SpellID]["Acquire"] = {}

	-- Assumption that there will only be 1 speciality for a trade skill
	RecipeDB[SpellID]["Specialty"] = Specialty or nil

end

-- Description: Adds all flag related information to the RecipeDB associated with the spell ID
-- Expected result: The RecipeDB will have flag information added for a specific recipe
-- Input: Recipe array reference, SpellID of recipe, all flags
-- Output: None, array is passed as a reference

function addon:addTradeFlags(RecipeDB, SpellID, ...)

	-- flags are defined in Documentation.lua

	local numvars = select('#',...)

	-- Find out how many flags we're adding
	for i=1,numvars,1 do

		-- Get the value of the current flag
		local flag = select(i, ...)

		RecipeDB[SpellID]["Flags"][flag] = true

	end
	

end

-- Description: Adds all Acquire related information to the RecipeDB associated with the spell ID
-- Expected result: The RecipeDB will have acqusition information added for a specific recipe
-- Input: Recipe array reference, SpellID of recipe, all Acquire info
-- Output: None, array is passed as a reference

function addon:addTradeAcquire(RecipeDB, SpellID, ...)

	-- Find out how many flags we're adding
	local numvars = select('#',...)

	-- Index for the number of Acquire entries we have
	local index = 1

	-- Index for which variables we're parsing through
	local i = 1

	while (i < numvars) do
		-- Create the space for the current Acquire method
		RecipeDB[SpellID]["Acquire"][index] = {}

		-- Get the Type and ID of the values
		local AcquireType, AcquireID = select(i, ...)

		RecipeDB[SpellID]["Acquire"][index]["Type"] = AcquireType
		RecipeDB[SpellID]["Acquire"][index]["ID"] = AcquireID

		i = i + 2

		if (AcquireType == 6) then
			local RepLevel, RepVendor = select(i, ...)
			RecipeDB[SpellID]["Acquire"][index]["RepLevel"] = RepLevel
			RecipeDB[SpellID]["Acquire"][index]["RepVendor"] = RepVendor
			i = i + 2
		end

		index = index + 1
	end
end

-- Description: Adds a specific entry (ie: vendor, mob, etc) to the lookup list
-- Expected result: Look up list will have an entry added to it
-- Input: List to add to, ID of entry, Name, Faction Location and Coordinates (X and Y)
-- Output: None, array is passed as a reference

function addon:addLookupList(DB, ID, Name, Loc, Coordx, Coordy, Faction)

	--[[
		For individual database structures, see Documentation.lua
	]]--

	DB[ID] = {}
	DB[ID]["Name"] = Name

	if (Loc) then

		DB[ID]["Location"] = Loc

	end

	if (Coordx) and (Coordy) then

		DB[ID]["Coordx"] = Coordx
		DB[ID]["Coordy"] = Coordy

	end

	if (Faction) then

		if (Faction == 1) then

			DB[ID]["Faction"] = BFAC["Horde"]

		elseif (Faction == 2) then

			DB[ID]["Faction"] = BFAC["Alliance"]

		elseif (Faction == 3) then

			DB[ID]["Faction"] = BFAC["Neutral"]

		end

	end

end

--[[

	Recipe Scanning Functions

]]--

-- Description: Scans the recipe listing and marks known recipes as true in the database
-- Expected result: The array of Recipes will have all known recipes toggles to true
-- Input: Recipe Array
-- Output: The total number of recipes known

function addon:ScanForKnownRecipes(RecipeDB, playerdata)

	self:Print("DEBUG: Scanning for known recipes.")

	-- Clear the "Have Materials" check box
	if not Skillet and TradeSkillFrameAvailableFilterCheckButton:GetChecked() then

		TradeSkillFrameAvailableFilterCheckButton:SetChecked(false)
		TradeSkillOnlyShowMakeable(false)

	end

	-- Clear the sub-classes filters
	SetTradeSkillSubClassFilter(0, 1, 1)
	UIDropDownMenu_SetSelectedID(TradeSkillSubClassDropDown, 1)

	-- Clear the inventory slot filter
	SetTradeSkillInvSlotFilter(0, 1, 1);
	UIDropDownMenu_SetSelectedID(TradeSkillInvSlotDropDown, 1)

	-- Expand all headers so we can see all the recipes there are
	for i = GetNumTradeSkills(), 1, -1 do

		local _, tradeType = GetTradeSkillInfo(i)

		if tradeType == "header" then

			ExpandTradeSkillSubClass(i)

		end

	end

	playerdata.foundRecipes = 0

	-- Scan throug\h all recipes
	for i = 1, GetNumTradeSkills() do

		local tradeName, tradeType = GetTradeSkillInfo(i)

		-- Ignore all trade skill headers
		if (tradeType ~= "header") then

			-- Get the trade skill link for the specified recipe
			local SpellLink = GetTradeSkillRecipeLink(i)
			-- Get the SpellID from the spell link
			local _,_,SpellString = sfind(SpellLink, "enchant:(%d+)")
			local SpellID = tonumber(SpellString)

			-- Spell ID is in RecipeDB so lets flag it as known
			if (RecipeDB[SpellID]) then

				-- Update array that recipe was found
				RecipeDB[SpellID]["Known"] = true
				playerdata.foundRecipes = playerdata.foundRecipes + 1

			-- We didn't find it in our database, lets notify people that we don't have it
			else

				self:Print(self:Red(tradeName .. " " .. SpellString) .. self:White(L["MissingFromDB"]))	

			end

		end

	end

end

do

	local classmap = {

		["DEATHKNIGHT"] = 21,
		["DRUID"] = 22,
		["HUNTER"] = 23,
		["MAGE"] = 24,
		["PALADIN"] = 25,
		["PRIEST"] = 26,
		["SHAMAN"] = 27,
		["ROGUE"] = 28,
		["WARLOCK"] = 29,
		["WARRIOR"] = 30

	}

	-- Description: Scans a specific recpie to determine if it is to be displayed or not.
	-- Expected result: Returns true or false depending on if the recipe should be displayed
	-- Input: 
	-- Output: Boolean on whether or not to display the recipe

	function addon:CheckDisplayRecipe(Recipe, AllSpecialtiesTable, playerProfessionLevel, playerProfession, playerSpecialty, playerFaction, playerClass)

		-- For flag info see comments at start of file in comments
		local filterdb = addon.db.profile.filters

		-- See Documentation file for logic explanation
	--[[
			item = {									-- Item Filters (Armor/Weapon)
				armor = {
					cloth		= true [ | false ]				-- Display cloth recipes
					leather	= true [ | false ]				-- Display leather recipes
					mail		= true [ | false ]				-- Display mail recipes
					plate		= true [ | false ]				-- Display plate recipes
					trinket	= true [ | false ]				-- Display trinket recipes
					cloak		= true [ | false ]				-- Display cloak recipes
					ring		= true [ | false ]				-- Display ring recipes
					necklace	= true [ | false ]				-- Display necklace recipes
				},
				weapon = {
					onehand	= true [ | false ]				-- Display one hand weapon recipes
					twohand	= true [ | false ]				-- Display two hand weapon recipes
					axe		= true [ | false ]				-- Display axe weapon recipes
					sword	= true [ | false ]				-- Display sword weapon recipes
					mace		= true [ | false ]				-- Display mace weapon recipes
					polearm	= true [ | false ]				-- Display polearm weapon recipes
					dagger	= true [ | false ]				-- Display dagegr weapon recipes
					fist		= true [ | false ]				-- Display fist weapon recipes
					staff		= true [ | false ]				-- Display staff weapon recipes
					wand		= true [ | false ]				-- Display wand weapon recipes
					thrown	= true [ | false ]				-- Display thrown weapon recipes
					bow		= true [ | false ]				-- Display bow weapon recipes
					crossbow	= true [ | false ]				-- Display crossbow weapon recipes
					ammo		= true [ | false ]				-- Display ammo recipes
				},
			},
			binding = {									-- Item/Recipe Binding filters
				itemboe		= true [ | false ]				-- Crafted Item is BoE
				itembop		= true [ | false ]				-- Crafted Item is BoP
				recipebop		= true [ | false ]				-- Recipe is BoE
				recipeboe		= true [ | false ]				-- Recipe is BoE
			},
			player = {									-- Player Type filters
				melee			= true [ | false ]				-- Melee DPS recipes
				tank			= true [ | false ]				-- Tank recipes
				healer		= true [ | false ]				-- Healer recipes
				caster		= true [ | false ]				-- Caster DPS recipes
			},
			rep = {									-- Reputation filters
													-- Azeroth faction filters
				aldor				= true [ | false ]
				scryer			= true [ | false ]
				argentdawn			= true [ | false ]
				ashtonguedeathsworn 	= true [ | false ]
				cenarioncircle		= true [ | false ]
													-- Outland faction filters
				cenarionexpedition	= true [ | false ]
				consortium			= true [ | false ]
				hellfire			= true [ | false ]
				keepersoftime		= true [ | false ]
				nagrand			= true [ | false ]
				lowercity			= true [ | false ]
				scaleofthesands		= true [ | false ]
				shatar			= true [ | false ]
				shatteredsun		= true [ | false ]
				sporeggar			= true [ | false ]
				thoriumbrotherhood	= true [ | false ]
				timbermaw			= true [ | false ]
				violeteye			= true [ | false ]
				zandalar			= true [ | false ]
													-- WotLK faction filters
				argentcrusade		= true [ | false ]
				frenzyheart		= true [ | false ]
				ebonblade			= true [ | false ]
				kirintor			= true [ | false ]
				sonsofhodir			= true [ | false ]
				kaluak			= true [ | false ]
				oracles			= true [ | false ]
				wyrmrest			= true [ | false ]
				silvercovenant		= true [ | false ]
				sunreavers			= true [ | false ]
				explorersleague		= true [ | false ]
				valiance			= true [ | false ]
				handofvengeance		= true [ | false ]
				taunka			= true [ | false ]
				warsongoffensive		= true [ | false ]
			}
		}
	************************************************************************
	]]--

--[[
************************************************************************
Item Flags
	A recipe can be flagged with many (or even all) of the below flags. They are defined
	as follows:

	-- Type of crafted item (36-45 for growth):
		-- 36 = Item BoE
		-- 37 = Item BoP
		-- 38 = Item BoA
		-- 40 = Recipe BoE
		-- 41 = Recipe BoP
		-- 42 = Recipe BoA

	-- Item class (46-50 for growth):
		-- 46 = Weapon
		-- 47 = Armor

	-- Item Type (51-55 for growth):
		-- 51 = Physical DPS (melee/hunters)
		-- 52 = Tanking
		-- 53 = Healing
		-- 54 = Caster DPS

	-- Armor Type (56-65 for growth):
		-- 56 = Cloth
		-- 57 = Leather
		-- 58 = Mail
		-- 59 = Plate
		-- 60 = Cloak
		-- 61 = Trinket
		-- 62 = Ring
		-- 63 = Necklace
		-- 64 = Shield

	-- Weapon Type (66-85 for growth):
		-- 66 = 1H
		-- 67 = 2H
		-- 68 = Axe
		-- 69 = Sword
		-- 70 = Mace
		-- 71 = Polearm
		-- 72 = Dagger
		-- 73 = Staff
		-- 74 = Wand
		-- 75 = Thrown
		-- 76 = Bow
		-- 77 = Crossbow
		-- 78 = Ammo
		-- 79 = Fist

	-- Reputation
		-- Original WoW (96-100):
			-- 96 = Argent Dawn
			-- 97 = Cenarion Circle
			-- 98 = Thorium Brotherhood
			-- 99 = Timbermaw Hold
			-- 100 = Zandalar Tribe
		-- Burning Crusade (101-114):
			-- 101 = The Aldor
			-- 102 = Ashtongue Deathsworn
			-- 103 = Cenarion Expedition
			-- 104 = Hellfire Factions (Thrallmar/Honor Hold)
			-- 105 = Consortium
			-- 106 = Keepers of Time
			-- 107 = Lower City
			-- 108 = Nagrand Factions (Maghar/Kureni)
			-- 109 = Scale of the Sands
			-- 110 = The Scryer
			-- 111 = Sha'tar
			-- 112 = Shattered Sun
			-- 113 = Sporeggar
			-- 114 = Violeteye
		-- Below this is WotLK stuff. May need to be changed post beta
		-- LK (115-XXX):
			-- 115 = Argent Crusade
			-- 116 = Frenzyheart Tribe
			-- 117 = Knights of the Ebon Blade
			-- 118 = Kirin Tor
			-- 119 = The Sons of Hodir
			-- 120 = The Kalu'ak
			-- 121 = The Oracles
			-- 122 = The Wyrmrest Accord
			-- 123 = The Silver Convenant
			-- 124 = The Sunreavers
			-- 125 = Explorer's League
			-- 126 = Valiance Expedition
			-- 127 = The Hand of Vengeance
			-- 128 = The Taunka
			-- 129 = Warsong Offensive

************************************************************************
--]]

		-- Stage 1
		-- Loop through exclusive flags (hard filters)
		-- If one of these does not pass we do not display the recipe
		-- So to be more effecient we'll just leave this function if there's a false

		-- Include known
		if ((filterdb.general.known == false ) and (Recipe["Known"] == true )) then
			return false
		end

		-- Include unknown
		if (filterdb.general.unknown == false) and (Recipe["Known"] == false) then
			return false
		end

		-- Is this recipe in my currently selected profession?
		if (Recipe["Profession"] ~= playerProfession) then
			return false
		end

		-- Display both horde and alliance factions?
		if (filterdb.general.faction == false) then
			-- We want to filter out all the Horde only recipes
			if (playerFaction == BFAC["Alliance"]) then
				-- Filter out Horde only
				if (Recipe["Flags"][1] == false) and (Recipe["Flags"][2] == true) then
					return false
				end
			-- We want to filter out all the Alliance only recipes
			else
				-- Filter out Alliance only
				if (Recipe["Flags"][2] == false) and (Recipe["Flags"][1] == true) then
					return false
				end
			end
		end

		-- Display all skill levels?
		if (filterdb.general.skill == false) then
			if (Recipe["Level"] > playerProfessionLevel) then
				return false
			end
		end

		-- Display all classes?
		if (filterdb.general.class == true) then
			if (Recipe["Flags"][classmap[playerClass]] == false) then
				return false
			end
		end

		-- Display all specialities?
		if (filterdb.general.specialty == true) then
			if (Recipe["Specialty"] ~= playerSpecialty) then
				return false
			end
		end

		-- Stage 2
		-- loop through nonexclusive (soft filters) flags until one is true
		-- If one of these is true (ie: we want to see trainers and there is a trainer flag) we display the recipe

		-- Display trainer recipes
		if (filterdb.obtain.trainer == true) then
			if (Recipe["Flags"][3] == true) then
				return true
			end
		end

		-- Display vendor recipes
		if (filterdb.obtain.vendor == true) then
			if (Recipe["Flags"][4] == true) then
				return true
			end
		end

		-- Display instance recipes
		if (filterdb.obtain.instance == true) then
			if (Recipe["Flags"][5] == true) then
				return true
			end
		end

		-- Display raid recipes
		if (filterdb.obtain.raid == true) then
			if (Recipe["Flags"][6] == true) then
				return true
			end
		end

		-- Display seasonal recipes
		if (filterdb.obtain.seasonal == true) then
			if (Recipe["Flags"][7] == true) then
				return true
			end
		end

		-- Display trainer recipes
		if (filterdb.obtain.quest == true) then
			if (Recipe["Flags"][8] == true) then
				return true
			end
		end

		-- Display trainer recipes
		if (filterdb.obtain.PVP == true) then
			if (Recipe["Flags"][9] == true) then
				return true
			end
		end

		-- Display trainer recipes
		if (filterdb.obtain.worlddrop == true) then
			if (Recipe["Flags"][10] == true) then
				return true
			end
		end

		-- Display trainer recipes
		if (filterdb.obtain.mobdrop == true) then
			if (Recipe["Flags"][11] == true) then
				return true
			end
		end

		-- Display trainer recipes
		if (filterdb.obtain.discovery == true) then
			if (Recipe["Flags"][12] == true) then
				return true
			end
		end

		-- If we get here it means that no flags matched our values
		self:Print("DEBUG: No filter flags matched")
		return true

	end

end

-- Description: Creates an array of which factions we want to include in our display and which ones to ignore
-- Expected result: Array of all factions created with those which we want to display having a true flag
-- Input: Rep Table Array
-- Output: None, array is passed as a reference

function addon:PopulateRepFilters(RepTable)

	self:Print("DEBUG: Updating Rep Filters")
	local repfilters = addon.db.profile.filters.rep
	RepTable[BFAC["The Scryers"]] = repfilters.scryer
	RepTable[BFAC["The Aldor"]] = repfilters.aldor
	RepTable[BFAC["Argent Dawn"]] = repfilters.argentdawn
	RepTable[BFAC["Ashtongue Deathsworn"]] = repfilters.ashtonguedeathsworn
	RepTable[BFAC["Cenarion Circle"]] = repfilters.cenarioncircle
	RepTable[BFAC["Cenarion Expedition"]] = repfilters.cenarionexpedition
	RepTable[BFAC["The Consortium"]] = repfilters.consortium
	RepTable[BFAC["Honor Hold"]] = repfilters.hellfire
	RepTable[BFAC["Thrallmar"]] = repfilters.hellfire
	RepTable[BFAC["Keepers of Time"]] = repfilters.keepersoftime
	RepTable[BFAC["Kurenai"]] = repfilters.nagrand
	RepTable[BFAC["The Mag'har"]] = repfilters.nagrand
	RepTable[BFAC["Lower City"]] = repfilters.lowercity
	RepTable[BFAC["The Scale of the Sands"]] = repfilters.scaleofthesands
	RepTable[BFAC["The Sha'tar"]] = repfilters.shatar
	RepTable[BFAC["Shattered Sun Offensive"]] = repfilters.shatteredsun
	RepTable[BFAC["Sporeggar"]] = repfilters.sporeggar
	RepTable[BFAC["Thorium Brotherhood"]] = repfilters.thoriumbrotherhood
	RepTable[BFAC["Timbermaw Hold"]] = repfilters.timbermaw
	RepTable[BFAC["The Violet Eye"]] = repfilters.violeteye
	RepTable[BFAC["Zandalar Tribe"]] = repfilters.zandalar
-- Below this is WotLK data. May need to be changed post beta.
	RepTable[BFAC["Argent Crusade"]] = repfilters.argentcrusade
	RepTable[BFAC["Frenzyheart Tribe"]] = repfilters.frenzyheart
	RepTable[BFAC["Knights of the Ebon Blade"]] = repfilters.ebonblade
	RepTable[BFAC["Kirin Tor"]] = repfilters.kirintor
	RepTable[BFAC["The Sons of Hodir"]] = repfilters.sonsofhodir
	RepTable[BFAC["The Kalu'ak"]] = repfilters.kaluak
	RepTable[BFAC["The Oracles"]] = repfilters.oracles
	RepTable[BFAC["The Wyrmrest Accord"]] = repfilters.wyrmrest
	RepTable[BFAC["The Silver Convenant"]] = repfilters.silvercovenant
	RepTable[BFAC["The Sunreavers"]] = repfilters.sunreavers
	RepTable[BFAC["Explorer's League"]] = repfilters.explorersleague
	RepTable[BFAC["Valiance Expedition"]] = repfilters.valiance
	RepTable[BFAC["The Hand of Vengeance"]] = repfilters.handofvengeance
	RepTable[BFAC["The Taunka"]] = repfilters.taunka
	RepTable[BFAC["Warsong Offensive"]] = repfilters.warsongoffensive
end

-- Description: Scans the recipe listing and updates the filters according to user preferences
-- Expected result: The array of Recipes will have all Display flags toggles according to display preferences and the number of filtered recipes
-- Input: Recipe Array, Skill level for current profession, name of current profession, and current profession Specialty
-- Output: Number of recipes that are filtered

function addon:UpdateFilters(RecipeDB, AllSpecialtiesTable, playerdata)

	local playerProfessionLevel = playerdata.playerProfessionLevel
	local playerProfession = playerdata.playerProfession
	local playerSpecialty = playerdata.playerSpecialty
	local playerFaction = playerdata.playerFaction
	local playerClass = playerdata.playerClass

	playerdata.filteredRecipes = 0

	-- Parse through all the entries in the Recipe array
	for RecipeID in pairs(RecipeDB) do

		-- Determine if we are to display this recipe or not
		local displayflag = self:CheckDisplayRecipe(RecipeDB[RecipeID], AllSpecialtiesTable, playerProfessionLevel, playerProfession, playerSpecialty, playerFaction, playerClass)

		if (displayflag == false) then

			playerdata.filteredRecipes = playerdata.filteredRecipes + 1

		end

		-- Set the display flag
		RecipeDB[RecipeID]["Display"] = displayflag

	end

end

--[[

	ARL Logic Functions

]]--

-- Description: Determines which profession we are dealing with and loads up the recipe information for it.
-- Expected result: The RecipeDBis populated with recipe information
-- Input: RecipeDB, Current Profession we're examining
-- Output: Total number of recipes in the database

local function InitializeRecipes(RecipeDB, playerdata)

	-- Table of all possible professions with init functions
	local professiontable =
	{
		[GetSpellInfo(2259)] = addon.InitAlchemy,
		[GetSpellInfo(2018)] = addon.InitBlacksmithing,
		[GetSpellInfo(2550)] = addon.InitCooking,
		[GetSpellInfo(7411)] = addon.InitEnchanting,
		[GetSpellInfo(4036)] = addon.InitEngineering,
		-- Use first aid spell of applying bandages to fix issues with other localizations
		[GetSpellInfo(746)] = addon.InitFirstAid,
		-- Hack to get first aid working on frFR since I can't seem to get a proper spell ID :P
		["Premiers soins"] = addon.InitFirstAid,
		--[GetSpellInfo(3273)] = addon.InitFirstAid,
		[GetSpellInfo(2108)] = addon.InitLeatherworking,
		[GetSpellInfo(2575)] = addon.InitSmelting,
		[GetSpellInfo(3908)] = addon.InitTailoring,
		[GetSpellInfo(25229)] = addon.InitJewelcrafting,
		[GetSpellInfo(45357)] = addon.InitInscription,
		[GetSpellInfo(28481)] = addon.InitRuneforging,
	}

	-- Thanks to sylvanaar/xinhuan for the code snippet
	local a = professiontable[playerdata.playerProfession]

	if a then
		playerdata.totalRecipes = a(addon, RecipeDB)
	else
		addon:Print(L["UnknownTradeSkill"]:format(playerdata.playerProfession))
	end
end

-- Description: Determines what to do when the slash command is called.
-- Expected result: The appropiate window is opened based on the slash command.
-- Input: None
-- Output: None

function addon:ChatCommand(input)

	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame.AckisRecipeList)
--[[
	if (not input) or (input and input:trim() == "") or (input == string.lower(L["About"])) then
		-- Open About panel if there's no parameters
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["about"])
	elseif (input == string.lower(L["Sort"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame[L["Sort"] ])
	elseif (input == string.lower(L["Filter"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame[L["Filter"] ])
	elseif (input == string.lower(L["Display"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["display"])
	elseif (input == string.lower(L["Profile"])) then
		InterfaceOptionsFrame_OpenToFrame(self.optionsFrame["profiles"])
	else
		LibStub("AceConfigCmd-3.0"):HandleCommand("arl", "Ackis Recipe List", input)
	end
]]--
end

do

	local RecipeList = nil

	local MobList = nil
	local QuestList = nil
	local ReputationList = nil
	local TrainerList = nil
	local SeasonalList = nil
	local VendorList = nil
	local RepFilters = nil
	local AllSpecialtiesTable = nil

	local playerdata = nil

	-- Description: Updates the reputation table.  This only happens seldomly so I'm not worried about effeciency
	-- Expected result: Reputation table is updated with appropiate levels
	-- Input: None
	-- Output: None

	function addon:SetRepDB()

		if (playerdata and playerdata["Reputation"]) then

			self:GetFactionLevels(playerdata["Reputation"])

		end

	end

	-- Description: Function called when the scan button is clicked.   Parses recipes and displays output
	-- Expected result: A gui window of all recipes accoring to filter is printed out
	-- Input: None
	-- Output: None

	function addon:AckisRecipeList_Command(textdump)

		-- If we don't have a trade skill window open, lets return out of here
		if (addon.SkillType == nil) then

			self:Print(L["OpenTradeSkillWindow"])
			return

		-- Trade type skills
		elseif (addon.SkillType == "Trade") then

			-- First time a scan has been run, we need to get the player specifc data, specifically faction information, profession information and other pertinant data.
			if (playerdata == nil) then

				playerdata = {}

				playerdata.playerFaction = UnitFactionGroup("player")
				local _
				_, playerdata.playerClass = UnitClass("player")

				playerdata["Reputation"] = {}
				self:GetFactionLevels(playerdata["Reputation"])

				playerdata["Professions"] = {
					[GetSpellInfo(2259)] = false, -- Alchemy
					[GetSpellInfo(2018)] = false, -- Blacksmithing
					[GetSpellInfo(2550)] = false, -- Cooking
					[GetSpellInfo(7411)] = false, -- Enchanting
					[GetSpellInfo(4036)] = false, -- Engineering
					[GetSpellInfo(746)] = false, -- First Aid
					--["Premiers soins"] = false, -- First Aid (Hack for frFR local)
					[GetSpellInfo(2108)] = false, -- Leatherworking
					[GetSpellInfo(2575)] = false, -- Smelting
					[GetSpellInfo(3908)] = false, -- Tailoring
					[GetSpellInfo(25229)] = false, -- Jewelcrafting
					[GetSpellInfo(45357)] = false, -- Inscription
					[GetSpellInfo(28481)] = false, -- Runeforging
				}

				self:GetKnownProfessions(playerdata["Professions"])

				-- All Alchemy Specialties
				local AlchemySpec = {
					[GetSpellInfo(28674)] = true,
					[GetSpellInfo(28678)] = true,
					[GetSpellInfo(28676)] = true,
				}

				-- All Blacksmithing Specialties
				local BlacksmithSpec = {
					[GetSpellInfo(9788)] = true, -- Armorsmith
					[GetSpellInfo(17041)] = true, -- Master Axesmith
					[GetSpellInfo(17040)] = true, -- Master Hammersmith
					[GetSpellInfo(17039)] = true, -- Master Swordsmith
					[GetSpellInfo(9787)] = true, -- Weaponsmith
				}

				-- All Engineering Specialties
				local EngineeringSpec = {
					[GetSpellInfo(20219)] = true, -- Gnomish
					[GetSpellInfo(20222)] = true, -- Goblin
				}

				-- All Leatherworking Specialties
				local LeatherworkSpec = {
					[GetSpellInfo(10657)] = true, -- Dragonscale
					[GetSpellInfo(10659)] = true, -- Elemental
					[GetSpellInfo(10661)] = true, -- Tribal
				}

				-- All Tailoring Specialties
				local TailorSpec = {
					[GetSpellInfo(26797)] = true, -- Spellfire
					[GetSpellInfo(26801)] = true, -- Shadoweave
					[GetSpellInfo(26798)] = true, -- Primal Mooncloth
				}

				-- List of classes which have Specialties
				local SpecialtyTable = {
					[GetSpellInfo(2259)] = AlchemySpec,
					[GetSpellInfo(2018)] = BlacksmithSpec,
					[GetSpellInfo(4036)] = EngineeringSpec,
					[GetSpellInfo(2108)] = LeatherworkSpec,
					[GetSpellInfo(3908)] = TailorSpec,
				}

				-- List containing all possible Specialties
				AllSpecialtiesTable = {}

				-- Populate the Specialty table with all Specialties, not adding alchemy because no recipes have alchemy filters
				for i in pairs(BlacksmithSpec) do AllSpecialtiesTable[i] = true end
				for i in pairs(EngineeringSpec) do AllSpecialtiesTable[i] = true end
				for i in pairs(LeatherworkSpec) do AllSpecialtiesTable[i] = true end
				for i in pairs(TailorSpec) do AllSpecialtiesTable[i] = true end

			end

			-- Initializes the mob list
			if (MobList == nil) then

				MobList = {}
				self:InitMob(MobList)

			end

			-- Initializes the quest list
			if (QuestList == nil) then

				QuestList = {}
				self:InitQuest(QuestList)

			end

			-- Initializes the reputation list
			if (ReputationList == nil) then

				ReputationList = {}
				self:InitReputation(ReputationList)

			end

			-- Initializes the trainer list
			if (TrainerList == nil) then

				TrainerList = {}
				self:InitTrainer(TrainerList)

			end

			-- Initializes the season list
			if (SeasonalList == nil) then

				SeasonalList = {}
				self:InitSeasons(SeasonalList)

			end

			-- Initializes the vendor list
			if (VendorList == nil) then

				VendorList = {}
				self:InitVendor(VendorList)

			end

			-- Initializes the reputation filters
			-- Don't assign values no because we do a scan later on
			if (RepFilters == nil) then

				RepFilters = {}

			end

			-- Initializes the recipe list
			-- Note there is no reset function implemented yet
			if (RecipeList == nil) then

				RecipeList = {}

			end

			-- Get the name of the current trade skill opened, along with the current level of the skill.
			playerdata.playerProfession, playerdata.playerProfessionLevel = GetTradeSkillLine()

			-- Get the current profession Specialty
			playerdata.playerSpecialty = self:GetTradeSpecialty(AllSpecialtiesTable, playerdata)

			-- Add the recipes to the database
			InitializeRecipes(RecipeList, playerdata)
			self:Print(playerdata.totalRecipes)

			-- Scan all recipes and mark the ones which ones we know
			self:ScanForKnownRecipes(RecipeList, playerdata)
			self:Print(playerdata.foundRecipes)

			-- Update the table containing which reps to display
			self:PopulateRepFilters(RepFilters)

			-- Add filtering flags to the recipes
			self:UpdateFilters(RecipeList, AllSpecialtiesTable, playerdata)
			self:Print(playerdata.filteredRecipes)

			-- Mark excluded recipes
			if ( not addon.db.profile.ignoreexclusionlist ) then
				self:GetExclusions(RecipeList)
			end
		end

		-- Sort the recipe list now
		local sortedindex = self:SortMissingRecipes(RecipeList)

		if (textdump == true) then

			local temptext = self:GetTextDump(RecipeList)

			self:DisplayTextDump(temptext)

		else

			self:CreateFrame( RecipeList, sortedindex, playerdata, AllSpecialtiesTable,
								TrainerList, VendorList, QuestList, ReputationList,
								SeasonalList, MobList )

		end

	end

end

--[[

	Sorting Functions

]]--

-- Description: Sorts the recipe Database depending on the settings defined in the database.
-- Expected result: A sorted array indexing values in the RecipeDB is returned.
-- Input: The Recipe Database
-- Output: A pointer to an array containing sorted values

function addon:SortMissingRecipes(RecipeDB)

	self:Print("DEBUG: Sorting recipe array")

	-- Create a new array for the sorted index
	local SortedRecipeIndex = {}

	-- Find out how he want to sort
	local sorttype = addon.db.profile.sorting

	-- Get all the indexes of the RecipeListing
	for n, v in pairs(RecipeDB) do
		tinsert(SortedRecipeIndex, n)
	end

	if (sorttype == L["Skill"]) then

		tsort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Level"] < RecipeDB[b]["Level"] end)

	elseif (sorttype == L["Name"]) then

		tsort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Name"] < RecipeDB[b]["Name"] end)

	elseif (sorttype == L["Acquisition"]) then

		tsort(SortedRecipeIndex, function(a,b) return RecipeDB[a]["Acquire"][1]["Type"] < RecipeDB[b]["Acquire"][1]["Type"] end)

	end

	return SortedRecipeIndex

end

--[[

	Recipe Exclusion Functions

--]]

-- Description: Marks all exclusions in the recipe database to not be displayed
-- Expected result: Parses the recipe database marking all exlusions to not be displays
-- Input: Recipe Database
-- Output: None, Recipe Database is passed as a reference

function addon:GetExclusions(RecipeDB)

	local exclusionlist = addon.db.profile.exclusionlist

	for i in pairs(exclusionlist) do
		-- We may have a recipe in the exclusion list that has not been scanned yet
		-- check if the entry exists in RecipeDB first
		if ( RecipeDB[i] ) then
			RecipeDB[i]["Display"] = false
		end
	end

end

-- Description: Removes or adds a recipe to the exclusion list.
-- Expected result: The exclusion database is updated.
-- Input: The spell IDsof the recipe
-- Output: Exclusion database is updated

function addon:ToggleExcludeRecipe(SpellID)

	local exclusionlist = addon.db.profile.exclusionlist

	-- Remove the Spell from the exclusion list
	if (exclusionlist[SpellID]) then

		exclusionlist[SpellID] = nil

	else

		exclusionlist[SpellID] = true

	end

end

--[[

	Searching Functions

]]--

-- Description: Scans through the recipe database and toggles the flag on if the item is in the search criteria
-- Expected result: The Search flag is toggle off for anything that does not meet search criteria
-- Input: Recipe database
-- Output: None, array is a reference

function addon:SearchRecipeDB(RecipeDB, searchstring)

	-- Go through the entire database
	for SpellID in pairs(RecipeDB) do

		-- Allow us to search by spell ID
		if sfind(SpellID,searchstring) or

		-- Allow us to search byitem ID
		sfind(RecipeDB[SpellID]["ItemID"],searchstring) or

		-- Allow us to search by name
		sfind(RecipeDB[SpellID]["Name"],searchstring) then

		-- Allow us to search in the Acquire information
		--sfind(RecipeDB[SpellID]["Name"],searchstring) then

			RecipeDB[SpellID]["Search"] = true

		else

			RecipeDB[SpellID]["Search"] = false

		end

	end

end

-- Description: Goes through the recipe database and resets all the search flags
-- Expected result: All search flags are reset to true
-- Input: Recipe database
-- Output: None, array is a reference

function addon:ResetSearch(RecipeDB)

	for SpellID in pairs(RecipeDB) do

		RecipeDB[SpellID]["Search"] = true

	end

end

--[[

	Text dumping functions

]]--

-- Description: Scans through the recipe database providing a string of comma seperated values for all recipe information
-- Expected result: Single string of all recipes provided.
-- Input: Recipe database
-- Output: Array is a reference, string of text dump

function addon:GetTextDump(RecipeDB)

	local texttable = {}

	for SpellID in pairs(RecipeDB) do

		if (RecipeDB[SpellID]["Known"]) then

			tinsert(texttable,SpellID .. "," .. RecipeDB[SpellID]["Name"] .. "," .. RecipeDB[SpellID]["Level"] .. "," .. ",true")

		else

			tinsert(texttable,SpellID .. "," .. RecipeDB[SpellID]["Name"] .. "," .. RecipeDB[SpellID]["Level"] .. "," .. ",false")

		end

	end

	return table.concat(texttable,"\n")

end
